// Header file for winebooker.cpp
//
// copyright 2003 Neil Barnes 
//
// nailed_barnacle@hotmail.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the license, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#ifndef __EMU__
#define __EMU__

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <windows.h>
#include <stdio.h>
#include <io.h>
#include <time.h>
#include "resource.h"


// seeing as there are wheel mice out there, it seems a shame to ignore them
// and as msvc 4.2 doesn't know about them, we bodge the message

#define		WM_WHEELMOUSE		522

PSTR 				szProgName = "Fiat Coupe EMU Monitor V0.3";    // application name
HINSTANCE		ghInstance;
HWND				hWnd;
PAINTSTRUCT ps;
HDC					hDC;

HWND				hEdit;



int PASCAL WinMain (HINSTANCE, HINSTANCE, LPSTR, int);
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM);

void SetComms (int baud);
unsigned char	Get16vEngineStatus (unsigned char ch, unsigned char noshow);
int Init20v (void);

// bosch routines
int initbosch (void);
int boschtx (unsigned char ch, int respond);
int boschrx (char * pch, int respond);
int frametx (unsigned char title, unsigned char length);
int framerx (void);


// some colour definitions

#define RED RGB(255,0,0)
#define GREEN RGB(0,255,0)
#define AMBER RGB(255,255,0)
#define MURPLE RGB(255,128,255)

static HBRUSH	hbRed;
static HBRUSH hbGreen;
static HBRUSH hbAmber;
static HBRUSH hbMurple;

#define TICKTOCK 1

enum _enginetype {NOTHING, _16VNA, _16VT, _20VNA, _20VT}; 

union	_16bit {
	unsigned short big;
	unsigned char little[2];
};

struct _engine {
	_16bit revs;
	_16bit injperiod;
	unsigned char advance;
	unsigned char vacuum;
	unsigned char airtemp;
	unsigned char watertemp;
	unsigned char throttle;
	unsigned char battery;
	unsigned char injtiming;
	unsigned char vae;
	unsigned char uniram1;
	unsigned char uniram2;
	unsigned char uniram3;
	unsigned char unieep1;
	unsigned char unieep2;
	unsigned char unieep3;
	unsigned char ecthrottle;
	unsigned char ecpressure;
	unsigned char eclambda;
	unsigned char ecwater;
	unsigned char ecknock;
	unsigned char ecair;
	unsigned char ecmix;
	unsigned char ecvae;
	unsigned char ecrpm;
	unsigned char ecram;
	unsigned char ecdigimatic;
	unsigned char ecnd;
	unsigned char eceeprom;
	unsigned char eceprom;
	unsigned char ecsequence;
	unsigned char eccam;
	unsigned char iso[6];
	unsigned char spare[11];
	unsigned char univas;
	unsigned char eevas;
	unsigned char crdvas;

		
};

struct _massage {
	float revs;
	float injperiod;
	float advance;
	float vacuum;
	float airtemp;
	float watertemp;
	float throttle;
	float battery;
	float injtiming;
	float vae;
	unsigned char uniram1;
	unsigned char uniram2;
	unsigned char uniram3;
};


// lookup table to translate the temperature value to degrees C
// for the 16v engine

int temperature[256] = {
	125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,
	125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,
	125,122,119,116,113,111,108,105,102, 99, 96, 93, 90, 87, 84, 82,
	 79, 77, 76, 75, 74, 73, 71, 70, 69, 68, 66, 65, 64, 63, 62, 60,
	 59, 58, 58, 57, 56, 56, 55, 54, 53, 53, 52, 51, 50, 50, 49, 48,
	 48, 47, 46, 46, 45, 45, 44, 43, 43, 42, 42, 41, 40, 40, 39, 39,
	 38, 37, 37, 36, 36, 35, 35, 34, 34, 33, 33, 32, 32, 31, 31, 30,
	 30, 29, 29, 28, 28, 27, 27, 26, 26, 25, 25, 24, 24, 23, 23, 22,
	 22, 22, 21, 21, 20, 20, 19, 19, 18, 18, 17, 17, 16, 16, 15, 15,
	 14, 14, 13, 13, 12, 12, 11, 11, 10, 10,  9,  9,  8,  8,  7,  7,
	  6,	6,  5,  5,  4,  4,  3,  3,  2,  2,  1,  1,  0,  0, -1, -1,
	 -2, -2, -3, -4, -4, -5, -6, -6, -7, -7, -8, -9, -9,-10,-11,-11,
	-12,-13,-14,-15,-16,-17,-18,-19,-20,-21,-22,-23,-24,-26,-27,-28,
	-29,-34,-35,-37,-39,-40,-42,-43,-45,-47,-48,-50,-52,-53,-55,-57,
	-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,
	-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55,-55 };

// and for the 20v engine
const char temperature20[256] = {
	-33,-33,-32,-31,-31,-30,-30,-29,-29,-28,-28, -27,-26,-25,-25,-24,
	-23,-23,-22,-22,-21,-20,-20,-19,-18,-18,-17,-16,-16,-15,-14,-13,
	-13,-12,-11,-11,-10,-10,-9,-8,-8,-7,-6,-6,-6,-5,-4,-3,
	-3,-2,-1,0,0,1,2,2,3,4,4,5,6,7,7,8,
	9,9,10,10,11,11,12,12,13,14,14,15,16,17,18,18,
	19,19,20,20,21,22,22,23,24,25,25,26,26,27,27,28,
	29,29,30,30,31,32,32,33,34,35,35,35,36,37,38,38,
	39,39,40,41,41,42,43,43,44,45,45,46,47,47,48,48,
	49,50,50,51,52,53,53,54,55,55,56,57,57,58,59,59,
	60,61,61,62,63,63,64,65,66,66,67,68,68,69,70,71,
	71,71,72,73,73,74,75,75,75,76,77,77,78,78,79,80,
	80,81,81,82,82,83,84,85,86,86,87,88,88,89,89,90,
	91,91,92,93,93,94,94,95,96,96,96,97,98,99,99,100,
	101,101,102,103,103,104,105,106,107,107,108,109,109,110,110,111,
	111,111,112,112,113,114,115,115,115,116,117,124,124,124,124,124,
	124,124,124,124,125,125,125,125,125,125,125,125,125,125,125,125 };


	

// a class to handle drawing the analogue displays in a pretty way

class Gauge {

public:

  Gauge (char * _title,			// what's the gauge called?
				char * _units,			// what units to display?
				int _height,				// how high is the display?
				int _x, 
				int _y,							// position of top left corner
				int _low,						// minimum display value
				int _high,					// maximum display value
				int _majortick,			// interval between labeled index marks
				int _minortick,			// interval between unlabeled index marks
				int _hieek,					// high danger level
				int _hiwarn,				// high warning level
				int _lowarn,				// low warning level (i.e. warn if below this value
				int _loeek,					// low danger level
				int places);				// number of decimal places in the dsplayed value

	void Show (HDC hDC, float value);
	void Update (HDC hDC, float value);
	int	Percentage (void);

	void Rescale(int _low,						// minimum display value
							int _high,					// maximum display value
							int _majortick,			// interval between labeled index marks
							int _minortick,			// interval between unlabeled index marks
							int _hieek,					// high danger level
							int _hiwarn,				// high warning level
							int _lowarn,				// low warning level (i.e. warn if below this value
							int _loeek);				// low danger level


private:

	char title[16];
	char units[16];
	float	value;				// the value we're currently displaying
	int height;				// height of the display, inc the title and value print
	int x;
	int y;						// location of the display
	int low;					// the lowest value on the scale
	int high;					// the highest value on the scale
	int majortick;		// the larger index mark spacing
	int minortick;		// the lesser index mark spacing (no values)
	int hieek;				// the value above which the value is considered dangerously high
	int hiwarn;				// the value above which a warning is necessary
	int lowarn;				// the value below which a warning is necessary
	int loeek;				// the value below which a warning is necessary
	int places;				// the number of decimal places to display


};

#endif
